﻿Module ApproximateStringMatching

    <System.Runtime.CompilerServices.Extension>
    Public Function Levenshtein(text1 As String, text2 As String) As Double
        Dim length1 = text1.Length
        Dim length2 = text2.Length

        Dim distance = New Integer(length1, length2) {}

        For x As Integer = 0 To length1
            distance(x, 0) = x
        Next
        For y As Integer = 0 To length2
            distance(0, y) = y
        Next

        For x As Integer = 1 To length1
            For y As Integer = 1 To length2
                Dim cost As Integer = If(text1(x - 1) = text2(y - 1), 0, 1)

                distance(x, y) = Min(distance(x - 1, y) + 1, distance(x, y - 1) + 1, distance(x - 1, y - 1) + cost)
            Next
        Next

        Return distance(length1, length2)
    End Function

    Private Function Min(a As Integer, b As Integer, c As Integer) As Integer
        Return Math.Min(a, Math.Min(b, c))
    End Function

    <System.Runtime.CompilerServices.Extension>
    Public Function Jaro(text1 As String, text2 As String) As Double
        Dim length1 As Double = text1.Length
        Dim length2 As Double = text2.Length

        ' zweimal in umgekehrter Reihenfolge, um Transpositionen zu ermitteln
        Dim common1 As String = CommonChars(text1, text2)
        Dim common2 As String = CommonChars(text2, text1)

        If common1.Length = 0 OrElse common1.Length <> common2.Length Then
            Return 0
        End If

        Dim common As Double = common1.Length

        Dim transpositions As Integer = CalcTranspositions(common1, common2)

        Dim similarity As Double = (common / length1 + common / length2 + (common - transpositions) / common) / 3.0
        Return If(Double.IsNaN(similarity), 0.0, similarity)
    End Function

    Private Function CommonChars(text1 As String, text2 As String) As String
        Dim length1 As Integer = text1.Length, length2 As Integer = text2.Length

        Dim halflen As Integer = Math.Min(length1, length2) / 2
        ' halbe Länge der kürzeren Zeichenkette
        Dim common = ""

        For x As Integer = 0 To length1 - 1
            For y As Integer = Math.Max(0, x - halflen) To Math.Min(x + halflen, length2) - 1
                If text1(x) = text2(y) Then
                    common += text1(x)
                    Exit For
                End If
            Next
        Next

        Return common
    End Function

    Private Function CalcTranspositions(common1 As String, common2 As String) As Integer
        Dim transpositions As Integer = 0

        For pos As Integer = 0 To common1.Length - 1
            If common1(pos) <> common2(pos) Then
                transpositions += 1
            End If
        Next

        Return transpositions / 2
    End Function


End Module
