﻿Public Enum TraverseModeEnum
    PreOrder
    PostOrder
    InOrder
    ReverseInOrder
End Enum

Public Class BinaryTree(Of TKey As IComparable(Of TKey), TValue)

    Private NotInheritable Class Node
        Public Property Key As TKey
        Public Property Value As TValue
        Public Property Left As Node
        Public Property Right As Node
    End Class

    Public Property TraverseMode As TraverseModeEnum

    Public ReadOnly Property Count As Integer
        Get
            Return _count
        End Get
    End Property
    Private _count As Integer

    Private root As Node

    Public Sub Add(key As TKey, value As TValue)
        If root Is Nothing Then
            root = New Node() With {.Key = key, .Value = value}
        Else
            AddTo(root, key, value)
        End If

        _count += 1
    End Sub
    Private Sub AddTo(node As Node, key As TKey, value As TValue)
        Dim c As Integer = key.CompareTo(node.Key)

        If c < 0 Then
            If node.Left Is Nothing Then
                node.Left = New Node() With {.Key = key, .Value = value}
            Else
                AddTo(node.Left, key, value)
            End If
        Else
            If node.Right Is Nothing Then
                node.Right = New Node() With {.Key = key, .Value = value}
            Else
                AddTo(node.Right, key, value)
            End If
        End If
    End Sub

    Public Function Contains(key As TKey)

        Dim node As Node = root
        While node IsNot Nothing
            Dim c As Integer = key.CompareTo(node.Key)

            If c = 0 Then
                Return True
            ElseIf c < 0 Then
                node = node.Left
            Else
                node = node.Right
            End If
        End While
        Return False
    End Function
    Default Public ReadOnly Property Item(key As TKey) As TValue
        Get
            Dim node As Node = root
            While node IsNot Nothing
                Dim c As Integer = key.CompareTo(node.Key)

                If c = 0 Then
                    Return node.Value
                ElseIf c < 0 Then
                    node = node.Left
                Else
                    node = node.Right
                End If
            End While
            Throw New ArgumentException("Schlüssel nicht vorhanden")
        End Get
    End Property
    Public Overrides Function ToString() As String
        Dim s As String = ""
        Dim level As Integer = 0

        Traverse(root, level, s)

        Return s
    End Function

    Private Sub Traverse(node As Node, level As Integer, ByRef s As String)
        If node Is Nothing Then
            Return
        End If

        If TraverseMode = TraverseModeEnum.PreOrder Then
            s += "".PadLeft(level, " "c) + node.Key.ToString() + vbLf
            Traverse(node.Left, level + 2, s)
            Traverse(node.Right, level + 2, s)
        End If

        If TraverseMode = TraverseModeEnum.PostOrder Then
            Traverse(node.Left, level + 2, s)
            Traverse(node.Right, level + 2, s)
            s += "".PadLeft(level, " "c) + node.Key.ToString() + vbLf
        End If

        If TraverseMode = TraverseModeEnum.InOrder Then
            Traverse(node.Left, level + 2, s)
            s += "".PadLeft(level, " "c) + node.Key.ToString() + vbLf
            Traverse(node.Right, level + 2, s)
        End If

        If TraverseMode = TraverseModeEnum.ReverseInOrder Then
            Traverse(node.Right, level + 2, s)
            s += "".PadLeft(level, " "c) + node.Key.ToString() + vbLf
            Traverse(node.Left, level + 2, s)
        End If
    End Sub

End Class
