﻿Public Class Hashtable(Of TKey, TValue)
    Private Class Pair
        Public Property Key As TKey  ' key nicht nachträglich ändern
        Public Property Value As TValue

        Public Sub New(key As TKey, value As TValue)
            Me.Key = key
            Me.Value = value
        End Sub
    End Class

    Private ReadOnly items As ArrayList(Of Pair)()

    Public ReadOnly Property Count As Integer
        Get
            Return _count
        End Get
    End Property
    Private _count As Integer

    Public Sub New(Optional length As Integer = 0)
        length = If(length = 0, 500, length)
        ' keine Verdopplung, da eigentlich Daten in ArrayList
        length = CalcPrimeLength(length)

        items = New ArrayList(Of Pair)(length - 1) {}
    End Sub

    Public Sub Add(key As TKey, value As TValue)

        If ContainsKey(key) Then
            Throw New ArgumentException("Item already exist in collection.")
        End If

        Dim hash As Integer = GetHash(key)

        Dim list = items(hash)

        If list Is Nothing Then ' Liste mit Werten existiert noch nicht
            list = New ArrayList(Of Pair)()
            items(hash) = list
        End If

        list.Add(New Pair(key, value))

        _count += 1
    End Sub

    Public Sub Remove(key As TKey)

        Dim hash As Integer = GetHash(key)

        Dim list = items(hash)

        If list IsNot Nothing Then
            For i As Integer = 0 To list.Count - 1
                If list(i).Key.Equals(key) Then
                    list.RemoveAt(i)
                    _count -= 1
                    Exit For
                End If
            Next
        End If

    End Sub

    Default Public Property Item(key As TKey) As TValue
        Get
            Dim hash As Integer = GetHash(key)

            Dim list = items(hash)

            If list IsNot Nothing Then
                For i As Integer = 0 To list.Count - 1
                    If list(i).Key.Equals(key) Then
                        Return list(i).Value
                    End If
                Next
            End If
            Return Nothing
        End Get
        Set
            If Not ContainsKey(key) Then
                Add(key, Value)
            Else
                Update(key, Value)
            End If
        End Set
    End Property

    Public Function Contains(key As TKey) As Boolean
        Return ContainsKey(key)
    End Function

    Public Function ContainsKey(key As TKey) As Boolean
        Dim hash As Integer = GetHash(key)

        Dim list = items(hash)

        If list IsNot Nothing Then
            For i As Integer = 0 To list.Count - 1
                If list(i).Key.Equals(key) Then
                    Return True
                End If
            Next
        End If
        Return False
    End Function

    Public Iterator Function Values() As IEnumerable(Of TValue)
        For Each list As ArrayList(Of Pair) In items
            If list IsNot Nothing Then
                For i As Integer = 0 To list.Count - 1
                    Yield list(i).Value
                Next
            End If
        Next
    End Function

    Public Iterator Function Keys() As IEnumerable(Of TKey)
        For Each list As ArrayList(Of Pair) In items
            If list IsNot Nothing Then
                For i As Integer = 0 To list.Count - 1
                    Yield list(i).Key
                Next
            End If
        Next
    End Function

    Public Overrides Function ToString() As String
        Dim s As String = ""

        For i As Integer = 0 To items.Length - 1
            Dim list = items(i)

            For Each pair As Pair In list
                s += pair.Key.ToString() + "|" + pair.Value.ToString() + " -> "
            Next
        Next
        Return (s & Convert.ToString("Count: ")) + Count.ToString()
    End Function

    Private Sub Update(key As TKey, value As TValue)
        Dim hash As Integer = GetHash(key)

        Dim list = items(hash)

        If list IsNot Nothing Then
            For i As Integer = 0 To list.Count - 1
                If list(i).Key.Equals(key) Then
                    list(i).Value = value
                    Exit For
                End If
            Next
        End If
    End Sub

    Private Function CalcPrimeLength(length As Integer) As Integer
        While Not IsPrime(length)
            length += 1
        End While

        Return length
    End Function

    Private Function IsPrime(number As Integer) As Boolean
        For i As Integer = 2 To number / 2
            If number Mod i = 0 Then
                Return False
            End If
        Next
        Return True
    End Function

    Private Function GetHash(key As TKey) As Integer
        Return Math.Abs(key.GetHashCode()) Mod items.Length
    End Function

End Class
