﻿Public Class HashtableLinearProbing(Of TKey, TValue)
    Private Class Pair
        Public Property Key As TKey
            Get
                Return _key
            End Get
            Private Set
                _key = Value
            End Set
        End Property
        Private _key As TKey

        Public Property Value As TValue
        Public Property IsDeleted() As Boolean

        Public Sub New(key As TKey, value As TValue)
            Me.Key = key
            Me.Value = value
        End Sub
    End Class

    Private ReadOnly items As Pair()

    Public ReadOnly Property Count() As Integer
        Get
            Return _count
        End Get
    End Property
    Private _count As Integer

    Public Sub New(Optional length As Integer = 0)
        length = CalcPrimeLength(If(length = 0, 500 * 2, length * 2))
        items = New Pair(length - 1) {}
    End Sub

    Public Sub Add(key As TKey, value As TValue)
        If ContainsKey(key) Then
            Throw New ArgumentException("Item already exist.")
        End If

        ' Überlauf, kein weiteres Element kann aufgenommen werden
        If Count + 1 = items.Length Then
            Throw New ArgumentException("No more space in array.")
        End If

        Dim hash As Integer = Math.Abs(key.GetHashCode()) Mod items.Length

        While items(hash) IsNot Nothing AndAlso Not items(hash).IsDeleted
            hash += 1
            hash = hash Mod items.Length
        End While

        items(hash) = New Pair(key, value)

        _count += 1
    End Sub
    Public Sub Remove(key As TKey)
        Dim hash As Integer = Math.Abs(key.GetHashCode()) Mod items.Length

        While items(hash) IsNot Nothing
            If items(hash).Key.Equals(key) AndAlso Not items(hash).IsDeleted Then
                items(hash).IsDeleted = True
                _count -= 1
                Exit While
            End If
            hash += 1
            hash = hash Mod items.Length
        End While
    End Sub

    Default Public ReadOnly Property Item(key As TKey) As TValue
        Get
            Dim hash As Integer = Math.Abs(key.GetHashCode()) Mod items.Length

            While items(hash) IsNot Nothing
                If items(hash).Key.Equals(key) AndAlso Not items(hash).IsDeleted Then
                    Return items(hash).Value
                End If
                hash += 1
                hash = hash Mod items.Length
            End While
            Throw New ArgumentException("Key not found.")
        End Get
    End Property

    Public Function ContainsKey(key As TKey) As Boolean
        Dim hash As Integer = Math.Abs(key.GetHashCode()) Mod items.Length

        While items(hash) IsNot Nothing AndAlso Not items(hash).IsDeleted
            If items(hash).Key.Equals(key) Then
                Return True
            End If
            hash += 1
            hash = hash Mod items.Length
        End While
        Return False
    End Function

    Public Overrides Function ToString() As String
        Dim s As String = ""

        For i As Integer = 0 To items.Length - 1
            If items(i) IsNot Nothing AndAlso Not items(i).IsDeleted Then
                s += items(i).Key.ToString() + "|" + items(i).Value.ToString() + " -> "
            End If
        Next
        Return (s & Convert.ToString("Count: ")) + Count.ToString()
    End Function

    Private Function CalcPrimeLength(length As Integer) As Integer
        While Not IsPrime(length)
            length += 1
        End While

        Return length
    End Function

    Private Function IsPrime(number As Integer) As Boolean
        For i As Integer = 2 To number / 2
            If number Mod i = 0 Then
                Return False
            End If
        Next
        Return True
    End Function
End Class
