﻿Imports System.IO
Imports System.Net
Imports System.Net.Sockets

Public Class HttpServer
    Public Property LocalPath As String
    Public Property Port As Integer
    Private listener As TcpListener

    Public Sub New(port As Integer, path As String)
        Me.Port = port
        Me.LocalPath = path
    End Sub

    Public Sub Start()

        listener = New TcpListener(IPAddress.Loopback, Port)

        listener.Start()

        Dim requests = New Requests()
        While True
            Dim c = listener.AcceptTcpClient()

            requests.Add(c)

            If Not listener.Pending() Then
                requests.Process(LocalPath)
            End If
        End While

    End Sub

    Private Class Requests
        Private queue As New Queue(Of TcpClient)()

        Public Sub Add(c As TcpClient)
            queue.Enqueue(c)
        End Sub

        Public Sub Process(path As String)
            Console.WriteLine("Queue: " + queue.Count.ToString())

            While queue.Count > 0
                Try
                    Dim r = New Request(queue.Dequeue(), path)
                    Console.WriteLine("  " + r.LocalUrl)
                Catch e As Exception
                    Console.WriteLine("Error: " + e.Message)
                End Try
            End While
        End Sub

    End Class

    Private Class Request
        Private c As TcpClient
        Private localPath As String

        Public ReadOnly Property LocalUrl As String
            Get
                Return _localUrl
            End Get
        End Property
        Private _localUrl As String

        Public ReadOnly Property Verb As String
            Get
                Return _verb
            End Get
        End Property
        Private _verb As String

        Public ReadOnly Property HttpVersion As String
            Get
                Return _httpVersion
            End Get
        End Property
        Private _httpVersion As String

        Public ReadOnly Property Header As List(Of String)
            Get
                Return _header
            End Get
        End Property
        Private _header As List(Of String)

        Public Sub New(c As TcpClient, path As String)
            Me.c = c
            Me.localPath = path

            _header = New List(Of String)()

            Process()
        End Sub

        Private Sub Process()
            Dim r = New StreamReader(c.GetStream())
            Dim w = New StreamWriter(c.GetStream())

            Try
                HandleRequest(r, w)
            Catch e As Exception
                Failure(w)
                Throw e
            Finally
                w.Close()
                r.Close()
                c.Close()
            End Try
        End Sub

        Private Sub Success(stream As StreamWriter, ext As String)
            stream.WriteLine("HTTP/1.0 200 OK")
            stream.WriteLine(Convert.ToString("Content-Type: text/") & ext)
            stream.WriteLine("Connection: close")
            stream.WriteLine("")
        End Sub

        Private Sub Failure(stream As StreamWriter)
            stream.WriteLine("HTTP/1.0 404 not found")
            stream.WriteLine("Connection: close")
            stream.WriteLine("")
        End Sub

        Private Sub HandleRequest(r As StreamReader, w As StreamWriter)
            Dim request As String = r.ReadLine()

            If request IsNot Nothing Then
                Dim tokens As String() = request.Split(" "c)

                _verb = tokens(0).ToUpper()
                _localUrl = tokens(1)
                _httpVersion = tokens(2)

                ReadHeader(r)

                Dim f As String = System.IO.Path.Combine(System.IO.Path.GetFullPath(localPath), LocalUrl.TrimStart("/"c))

                Dim text As String = File.ReadAllText(f)

                Success(w, System.IO.Path.GetExtension(LocalUrl).TrimStart("."c))

                w.Write(text)
            End If
        End Sub

        Private Sub ReadHeader(r As StreamReader)
            Dim attribute As String = r.ReadLine()

            While attribute IsNot Nothing
                If attribute.Equals("") Then
                    Exit While
                End If
                Header.Add(attribute)
                attribute = r.ReadLine()
            End While
        End Sub

    End Class

End Class




