﻿Imports System.IO

Public Class OneR
    Private Class Item
        Public Property Attribute() As String
            Get
                Return _attribute
            End Get
            Private Set
                _attribute = Value
            End Set
        End Property

        Private _attribute As String
        Public Property Classification() As String
            Get
                Return _classification
            End Get
            Private Set
                _classification = Value
            End Set
        End Property
        Private _classification As String

        Public Property Frequency() As Integer

        Public Sub New(attribute As String, classification As String, Optional frequency As Integer = 1)
            Me.Classification = classification
            Me.Attribute = attribute
            Me.Frequency = frequency
        End Sub

        Public Overrides Function Equals(obj As Object) As Boolean
            Dim item As Item = TryCast(obj, Item)

            Return (Attribute.Equals(item.Attribute) AndAlso Classification.Equals(item.Classification))
        End Function

        Public Overrides Function GetHashCode() As Integer
            Return Classification.GetHashCode() + Attribute.GetHashCode()
        End Function
    End Class

    Private Class ItemSet
        Public Property Column() As String
            Get
                Return _column
            End Get
            Private Set
                _column = Value
            End Set
        End Property
        Private _column As String

        Public Property Items() As Hashtable(Of String, Item)
            Get
                Return _items
            End Get
            Private Set
                _items = Value
            End Set
        End Property
        Private _items As Hashtable(Of String, Item)

        Public Property ErrorRate() As Double
            Get
                Return _errorRate
            End Get
            Private Set
                _errorRate = Value
            End Set
        End Property
        Private _errorRate As Double

        Public Sub New(col As String)
            Column = col
            Items = New Hashtable(Of String, Item)(10)
        End Sub

        Public Sub AddItem(attribute As String, classification As String)
            Dim key = Convert.ToString(attribute & Convert.ToString("->")) & classification

            Dim item As Item = Nothing
            If Items.ContainsKey(key) Then
                item = Items(key)
            End If

            If item Is Nothing Then
                item = New Item(attribute, classification)
                Items(key) = item
            Else
                item.Frequency += 1
            End If
        End Sub

        Public Sub Process()
            Dim result = From item In Items.Values()
                         Order By item.Attribute, item.Frequency Descending
                         Select item

            Dim total As Integer = 0
            Dim correct As Integer = 0
            Dim attribute As String = Nothing

            For Each item As Item In result
                total += item.Frequency

                Dim key = item.Attribute + "->" + item.Classification

                ' Gruppenwechsel
                If attribute Is Nothing OrElse attribute <> item.Attribute Then
                    attribute = item.Attribute
                    correct += item.Frequency
                Else
                    Items.Remove(key)
                End If
            Next

            ErrorRate = 100.0 - (correct * 100.0 / total)
        End Sub

        Public Overrides Function ToString() As String
            Dim s As String = Column & Convert.ToString(vbLf)

            For Each value As Item In Items.Values()
                s += String.Format("  {0}->{1} : {2}" & vbLf, value.Attribute, value.Classification, value.Frequency)
            Next

            If Not Double.IsNaN(ErrorRate) Then
                s += String.Format(vbLf & "  ErrorRate: {0} %" & vbLf, ErrorRate)
            End If

            Return s
        End Function
    End Class

    Public Property Solution() As Hashtable(Of String, String)
        Get
            Return _solution
        End Get
        Private Set
            _solution = Value
        End Set
    End Property
    Private _solution As Hashtable(Of String, String)

    Public Sub New()
        Solution = New Hashtable(Of String, String)()
    End Sub

    Public Sub Build(r As TextReader)
        Dim list = New ArrayList(Of ItemSet)()

        Dim header = r.ReadLine().Split(New Char() {";"c})

        ' Spaltenüberschriften lesen
        For Each head As String In header
            list.Add(New ItemSet(head))
        Next

        ' Daten einlesen und Häufigkeiten ermitteln
        Dim line As String = r.ReadLine()

        While line IsNot Nothing
            Dim tokens = line.Split(New Char() {";"c})

            Dim classfication As String = tokens(0)

            For i As Integer = 1 To tokens.Length - 1
                Dim attribute As String = tokens(i)

                list(i).AddItem(attribute, classfication)
            Next
            line = r.ReadLine()
        End While
        r.Close()

        ' Gruppen mit bester Vorhersage pro Spalte ermitteln
        For Each items As ItemSet In list
            ' voher: Ausgabe alle Häufigkeitsverteilungen
            'Console.WriteLine(items);
            ' nacher: Ausgabe bester Häufigkeitsverteilungen
            'Console.WriteLine(items);
            items.Process()
        Next

        ' Spalte mit geringster Fehlerrate ermitteln
        Dim result = (From items In list Where Not Double.IsNaN(items.ErrorRate) Order By items.ErrorRate Select items)

        Dim sol As ItemSet = result.First()

        ' Lösung für Vorhersage speichern
        For Each item As Item In sol.Items.Values()
            Solution.Add(item.Attribute, item.Classification)
        Next

        Console.WriteLine("".PadLeft(80, "_"c))
        Console.WriteLine("Regeln: " + sol.ToString())

        list.Clear()
    End Sub

    Public Function Classify(value As String) As String
        Return Solution(value)
    End Function

End Class
