﻿Imports System.Drawing.Drawing2D
Imports System.Globalization
Imports System.IO

Public Class Viewer
    Private ReadOnly commands As New ArrayList(Of String())()

    Private ReadOnly p As New Pen(Color.Black)
    Private x1 As Integer = 1, y1 As Integer = 1

    Public Sub New(path As String)

        Using r As New StreamReader(path)
            Dim line As String = r.ReadLine()

            While line IsNot Nothing
                Dim tokens As String() = line.Split(New Char() {" "c}, StringSplitOptions.RemoveEmptyEntries)

                If tokens.Length > 0 Then
                    commands.Add(tokens)
                End If
                line = r.ReadLine()
            End While
        End Using
    End Sub

    Public Sub Paint(g As Graphics)
        Transform(g)

        For Each command As String() In commands
            Dim stack = New Stack(Of String)()

            For Each param As String In command
                stack.Push(param)
            Next

            Select Case stack.Pop().ToLower()
                Case "setlinewidth"
                    If True Then
                        p.Width = Integer.Parse(stack.Pop())
                    End If
                    Exit Select

                Case "setrgbcolor"
                    If True Then
                        ' amerikanische Schreibweise für Zahlen in PostScript
                        Dim ci = CultureInfo.InvariantCulture

                        Dim blue As Integer = CInt(Double.Parse(stack.Pop(), ci) * 255)
                        Dim green As Integer = CInt(Double.Parse(stack.Pop(), ci) * 255)
                        Dim red As Integer = CInt(Double.Parse(stack.Pop(), ci) * 255)

                        p.Brush = New SolidBrush(Color.FromArgb(red, green, blue))
                    End If
                    Exit Select

                Case "arc"
                    If True Then
                        Dim sweepAngle As Single = Single.Parse(stack.Pop())
                        Dim startAngle As Single = Single.Parse(stack.Pop())
                        Dim radius As Single = Single.Parse(stack.Pop())
                        Dim y As Single = Single.Parse(stack.Pop())
                        Dim x As Single = Single.Parse(stack.Pop())

                        g.DrawArc(p, x - radius, y - radius, radius * 2, radius * 2, startAngle,
                    sweepAngle)
                    End If
                    Exit Select

                Case "moveto"
                    If True Then
                        y1 = Integer.Parse(stack.Pop())
                        x1 = Integer.Parse(stack.Pop())
                    End If
                    Exit Select
                Case "lineto"
                    If True Then
                        Dim y2 As Integer = Integer.Parse(stack.Pop())
                        Dim x2 As Integer = Integer.Parse(stack.Pop())

                        g.DrawLine(p, x1, y1, x2, y2)

                        x1 = x2
                        y1 = y2
                    End If
                    Exit Select
            End Select
        Next
    End Sub

    Private Sub Transform(g As Graphics)
        ' kartesisches Koordinatensystem
        Dim matrix = New Matrix(1, 0, 0, -1, 0, 0)
        matrix.Translate(0, -Screen.PrimaryScreen.WorkingArea.Height - 100)
        g.Transform = matrix
    End Sub

End Class
