﻿Public Class LinkedList(Of T)
    Implements IEnumerable(Of T)


    Private NotInheritable Class Node
        Public Property Item As T
        Public Property [Next] As Node
    End Class

    Public Property Count() As Integer
        Get
            Return _count
        End Get
        Private Set
            _count = Value
        End Set
    End Property
    Private _count As Integer

    Public Overridable Sub Add(item As T)
        Dim newNode = New Node() With {
            .Item = item,
            .Next = Nothing
        }

        If first Is Nothing Then
            first = newNode
            last = newNode
        Else
            last.Next = newNode ' verketteter Knoten
            last = newNode  ' neuer letzter Knoten
        End If
        Count += 1

    End Sub

    Private first As Node, last As Node

    Default Public Property Item(index As Integer) As T
        Get
            Return FindByIndex(index).Item
        End Get
        Set
            FindByIndex(index).Item = Value
        End Set
    End Property

    Public Overridable Sub AddRange(items As T())
        For Each obj In items
            Add(obj)
        Next
    End Sub


    Public Function Contains(item As T) As Boolean
        Dim node = Find(item)
        If node Is Nothing Then
            Return False
        End If
        Return True
    End Function

    Public Function Remove(item As T) As Boolean
        Dim node As Node = Find(item)

        If node Is Nothing Then
            Return False
        End If

        Dim previousNode As Node = FindPrevious(item)

        If previousNode IsNot Nothing Then  ' aus Mitte oder Ende entfernen
            previousNode.Next = node.Next

            If Object.ReferenceEquals(node, last) = True Then
                last = previousNode
            End If
        Else
            first = node.Next               ' ersten entfernen, previousNode = null

            If first Is Nothing Then        ' Liste leer
                last = Nothing
            End If
        End If

        Count -= 1

        Return True
    End Function

    Public Sub Clear()
        first = Nothing
        last = Nothing
    End Sub

    Private Function Enumerator_GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
        Return GetEnumerator()
    End Function

    Public Iterator Function GetEnumerator() As IEnumerator(Of T) Implements IEnumerable(Of T).GetEnumerator
        Dim node As Node = first

        While node IsNot Nothing
            Yield node.Item
            node = node.Next
        End While

        ' Return New LinkedListEnumerator(Me)
    End Function

    Private Function FindPrevious(item As T) As Node
        Dim previousNode As Node = Nothing
        Dim node As Node = first

        While node IsNot Nothing
            If node.Item.Equals(item) Then
                Return previousNode
            End If

            previousNode = node
            node = node.Next
        End While
        Return Nothing
    End Function

    Private Function Find(item As T) As Node
        Dim node As Node = first

        While node IsNot Nothing
            If node.Item.Equals(item) Then
                Return node
            End If

            node = node.Next
        End While
        Return Nothing
    End Function

    Private Function FindByIndex(index As Integer) As Node
        If index >= Count Then
            Throw New IndexOutOfRangeException()
        End If

        Dim node As Node = first
        Dim i As Integer = 0

        While node IsNot Nothing
            If i = index Then
                Return node
            End If

            node = node.Next
            i += 1
        End While
        Return Nothing
    End Function

End Class

